# $Id: Rwhois.pm,v 1.5 1998/07/30 00:52:46 davidb Exp $

package Net::Rwhois;

=head1 NAME

Net::Rwhois -- Object-oriented Rwhois 1.0/1.5 client interface.

=head1 SYNOPSIS

  use Net::Rwhois;

  $client = new Net::Rwhois(Host  => "root.rwhois.net",
                            Port  => 4321);
  $client->open();

  $version = $client->get_server_version();

  @results = $client->execute_query(Query_String => "domain netsol.com");

  $client->write_line("-holdconnect on");

  ($response, $result) = $client->read_to_response();

  @results = $client->execute_query(Class => "domain",
                                    Value => "netsol.com")

  ($response, $result) = $client->read_to_response();

=head1 DESCRIPTION

C<Net::Rwhois> provides an object-oriented RWhois client API.  It is
mostly a wrapper around the subobjects that form the complete API.
The intention of this interface is to produce a simpler, useful API.

=head1 CONSTRUCTOR

=over 4

=item new( ARGS )

Creates a C<Net::Rwhois> object.  It takes arguments in key-value
pairs.  The valid arguments are "Host", "Port", and "Proto" (case
ignored). "Host" is required.  "Port" defaults to the IANA assigned
4321, and "Proto" defaults to 'tcp'.  These arguments are simply
passed on the the contained C<Net::Rwhois::Connection> object.

=back

=head1 METHODS

=over 4

=item configure ( ARGS )

Takes the same arguments as the constructor, with the same logic.

=item execute_query( ARGS )

executes a query against the server currently pointed to by the client
object (initially set via the constructor), and returns an array of
C<Net::Rwhois::RwhoisObject>s.  The method takes arguments in
key-value pairs, like C<Net::Rwhois::Query::new).  The tags recognized
area "Query_String", "Class_Name", "Attribute", "Value", and "Limit".
See C<Net::Rwhois::Query> for more details.

=item results_to_string( @results )

translates the list of RwhoisObjects to a single string, using the
same behavior as C<Net::Rwhois::RwhoisObject::to_string>.

=back

=head1 PASS-THROUGH METHODS

This class contains methods that are simply wrappers around the methods of the contained (has-a) objects.

=over 4

=item Net::Rwhois::Connection Methods

  open
  close
  get_version
  read_line
  read_to_response
  write_line

=back

=head1 SEE-ALSO

=over 4

=item L<Net::Rwhois::Connection>

=item L<Net::Rwhois::Client>

=item L<Net::Rwhois::Query>

=item L<Net::Rwhois::QueryResult>

=item L<Net::Rwhois::RwhoisObject>

=back

=cut


require 5.003;

use Carp;
use Net::Rwhois::Connection;
use Net::Rwhois::Client;
use Net::Rwhois::Query;
use Net::Rwhois::QueryResult;
use Net::Rwhois::RwhoisObject;

use strict;
use vars qw($VERSION);

$VERSION = "0.04";

sub new {
  my $this = shift;
  my $class = ref($this) || $this;
  my %args = @_;

  my $self = {};
  bless $self, $class;

  # set defaults
  $self->configure(\%args);

  $self;
}

sub configure {
  my $self = shift;
  my $args = shift;

  $self->{'connection'} = new Net::Rwhois::Connection(%$args);

}

sub execute_query {
  my $self = shift;
  my %args = @_;

  my $query = new Net::Rwhois::Query(%args);

  my $query_result = $query->execute($self->{'connection'});

  $query_result->read_results();

  $query_result->get_all_objects();
}

##
## Net::Rwhois::Connection Pass-Through Methdods
##

sub open {
  my $self = shift;

  $self->{'connection'}->open();
}

sub close {
  my $self = shift;

  $self->{'connection'}->close();
}

sub get_server_version {
  my $self = shift;


  $self->{'connection'}->get_version();
}

sub read_line {
  my $self = shift;

  $self->{'connection'}->read_line();
}


sub read_to_response {
  my $self = shift;

  $self->{'connection'}->read_to_response()
}

sub write_line {
  my $self = shift;
  my $line = shift;

  $self->{'connection'}->write_line($line);
}

##
## Net::Rwhois::RwhoisObject Pass-Through Methods
##

sub results_to_string {
  my $self = shift;
  my @results = @_;
  my $obj;

  my $string = "";
  my $r;

  for $obj (@results) {
    $r = $obj->to_string();
    $string .= $r;
    $string .= "\n";
  }

  $string;
}


